# -*- coding: utf-8 -*-
"""
Created on Fri Feb 12 08:37:32 2021

@author: Andreas
"""

import numpy as np
import meshio


def xdmf(history, filename="timeseries"):

    p = np.zeros(3)
    dX = history[0].GLO.cube.edges

    X = np.array(
        [
            p,
            p + dX[:, 0],
            p + dX[:, 0] + dX[:, 1],
            p + dX[:, 1],
            p + dX[:, 2],
            p + dX[:, 0] + dX[:, 2],
            p + dX[:, 0] + dX[:, 1] + dX[:, 2],
            p + dX[:, 1] + dX[:, 2],
        ]
    )

    pts = np.array(
        [
            p + dX[:, 1] / 2 + dX[:, 2] / 2,
            p + dX[:, 2] / 2 + dX[:, 0] / 2,
            p + dX[:, 0] / 2 + dX[:, 1] / 2,
            #
            p + dX[:, 1] / 2 + dX[:, 2] / 2 + dX[:, 0],
            p + dX[:, 2] / 2 + dX[:, 0] / 2 + dX[:, 1],
            p + dX[:, 0] / 2 + dX[:, 1] / 2 + dX[:, 2],
        ]
    )

    cells = [("hexahedron", np.arange(0, 8).reshape(1, 8))]
    verts = [("vertex", np.arange(0, 6).reshape(6, 1))]

    filename1 = filename + "_cube.xdmf"
    with meshio.xdmf.TimeSeriesWriter(filename1) as writer:
        writer.write_points_cells(X, cells)
        for i, h in enumerate(history):
            F = h.INT.gridvec.components
            u = (F @ X.T).T - X
            s = h.INT.cauchy
            lpf = h.EXT.lpf
            s6 = np.array([s[0, 0], s[1, 1], s[2, 2], s[0, 1], s[1, 2], s[2, 0]])
            writer.write_data(
                i,
                point_data={"Displacement": u},
                cell_data={
                    "Cauchy Stress": s6.reshape(1, 6),
                    "Load-Proportionality-Factor (LPF)": np.array([[lpf]]),
                },
            )

    filename2 = filename + "_points.xdmf"
    with meshio.xdmf.TimeSeriesWriter(filename2) as writer:
        writer.write_points_cells(pts, verts)
        for i, h in enumerate(history):
            F = h.INT.gridvec.components
            u = (F @ pts.T).T - pts

            r = h.INT.force.components
            t = h.INT.traction.components

            rr = np.vstack((-r.T, r.T))
            tt = np.vstack((-t.T, t.T))
            writer.write_data(
                i, point_data={"Displacement": u, "Reaction Force": rr, "Traction": tt}
            )

    return



# -*- coding: utf-8 -*-
"""
Created on Thu Feb 25 16:50:18 2021

@author: Andreas
"""

import matplotlib.pyplot as plt
import numpy as np

import pytest

import cubrium


def test_svk_ux_k2():
    ux = cubrium.loadcase.uniaxial
    base_svk(2, 1, 5000, 20, ux, 0)


def base_svk(k, mu, bulk, steps, lcase, ix):
    MDL = cubrium.init()

    MDL.GLO.constitution.matid = 1
    MDL.GLO.constitution.parameters = [mu, bulk, k]

    # import loadcase settings
    MDL = lcase(MDL)

    # Update Problem Definition with loadcase settings
    MDL = cubrium.update(MDL)

    # initial solution
    x0 = np.zeros(9)
    lpf0 = 0.0

    # numeric continuation
    Res = cubrium.solve(MDL)(
        x0=x0,
        lpf0=lpf0,
        dxmax=0.05,
        dlpfmax=0.05,
        control0=10,
        jacmode=3,
        jaceps=1e-4,
        maxsteps=steps,
        maxcycles=4,
        maxiter=20,
        tol=1e-10,
    )

    # compile results
    Y = np.array([res.x for res in Res])
    # F = np.array([res.fun for res in Res])
    # J = np.array([np.linalg.det(res.x[:-1].reshape(3,3)+np.eye(3)) for res in Res])

    # recover internal quantities
    history = cubrium.recover(Y, MDL)

    # xdmf time-series writer
    fname = MDL.GLO.title + "_mu={0:g}_K={1:g}_k={2:g}".format(
        *MDL.GLO.constitution.parameters
    )
    cubrium.writer.xdmf(
        history,
        filename=fname,
    )

    # plot results
    plt.figure()
    plt.plot(Y[:, ix], Y[:, -1], "-")
    plt.plot(Y[0, ix], Y[0, -1], "ko")

    comp = [(1, 1), (1, 2), (1, 3), (2, 1), (2, 2), (2, 3), (3, 1), (3, 2), (3, 3)]

    plt.xlabel("comp. %d%d of displacement gradient" % comp[ix])
    plt.ylabel("load-proportionality-factor LPF")

    plt.savefig(fname + ".svg")

    return


if __name__ == "__main__":
    ux = cubrium.loadcase.uniaxial
    bx = cubrium.loadcase.biaxial
    ps = cubrium.loadcase.planarshear

    sh = cubrium.loadcase.simpleshear
    sf = cubrium.loadcase.simpleshearfree2free3

    mu = 1
    bulk = 5000
    steps = 20

    kk = [2, 1, 0, -1]

    for k in kk:
        base_svk(k, mu, bulk, steps, sf, 1)
        base_svk(k, mu, bulk, steps, sh, 1)
        base_svk(k, mu, bulk, steps, ux, 0)
        base_svk(k, mu, bulk, steps, bx, 0)
        base_svk(k, mu, bulk, steps, ps, 0)



# -*- coding: utf-8 -*-
"""
Created on Fri Feb 12 08:29:52 2021

@author: adutz
"""

import numpy as np


def gridvecns(F):
    "Normal and shear components of gridvectors (edges of cube)."

    Fn = np.zeros(3)
    Fs = np.zeros((3, 3))
    for i in range(3):
        Fn[i] = np.linalg.norm(F[:, i])

    Fs = F.copy()
    # np.fill_diagonal(Fs,np.nan)

    return Fn, Fs


def defgrd(H):
    "Calculates deformation gradient from displacement gradient tensor."
    return np.eye(3) + H.reshape(3, 3)



from .__about__ import __version__

from . import assembly
from . import constitution
from . import helpers
from . import kinematics
from . import kinetics
from . import loadcase
from . import system
from . import writer

from .system import init
from .system import update

from .assembly import recover

from .solver import solve


__all__ = [
    "__version__",
]



# -*- coding: utf-8 -*-
"""
Created on Thu Feb 25 16:45:59 2021

@author: Andreas
"""

import numpy as np


def ddot(A, B):
    return np.tensordot(A, B, 2)


def dya(A, B):
    return np.tensordot(A, B, 0)


def cdya(A, B):
    return (np.einsum("ij,kl->ikjl", A, B) + np.einsum("ij,kl->ilkj", A, B)) / 2


def dev(A):
    return A - np.trace(A) / 3 * np.eye(3)



# -*- coding: utf-8 -*-
"""
Created on Fri Feb 26 10:50:00 2021

@author: adutz
"""

from functools import partial
import contique

from .assembly import equilibrium


def solve(MDL):
    return partial(contique.solve, fun=equilibrium, args=(MDL,))
